// Demo Sketch for 4x TI DRV2605 and I2C MUX
//
// Usage:
// Serial Connect w/ 9600 baud
// 0-3 send a buzz on channel 0-3 respectively

#include "Adafruit_DRV2605.h"

const uint8_t channelCount = 4; // belt has 3 channels
Adafruit_DRV2605 drvs[channelCount] = {
  Adafruit_DRV2605(0),  
  Adafruit_DRV2605(1),
  Adafruit_DRV2605(2),
  Adafruit_DRV2605(3)
};

uint8_t library = 6; // LRA
char motor = '!'; // '!' = disabled
bool debug = false;

void setup() {
  Wire.begin();
  
  // wait for serial port to connect
  while (!Serial); 
  Serial.begin(9600);
  delay(250);
  
  for (uint8_t i = 0; i < channelCount; i++) {
    drvs[i].begin();
    drvs[i].selectLibrary(library);
    drvs[i].useLRA();
    drvs[i].setMode(DRV2605_MODE_REALTIME);
  }
}

void loop() {
  if (Serial.available()) {
    
    char cmd = Serial.read();
    if (debug) {
      Serial.print("Received: ");
      Serial.println(cmd);
    }

    uint8_t intensity = 0;
    int duration = 0;

    switch (cmd) {
      if (debug) {
          Serial.print("playing on channel ");
          Serial.println(cmd);
      }
      case ('0'):

      // single motor: play 'rare' case
      case ('0'):
      case ('1'):
      case ('2'):
      case ('3'): // disabled for 3 motors
        motor = cmd;
        intensity = 255;
        duration = 200;
    }

    if (intensity > 0) {
      // single channel

      // convert ASCII channel string to int
      uint8_t channel = ((uint8_t)motor) - 48;

      if ((motor != '!') && (channel < channelCount)) {
        playRTP(channel, intensity, duration);
        motor = '!'; // reset to normal mode
      } else {
        playRTP(intensity, duration);
      }
      intensity = 0;
      duration = 0;
    }

    if (debug) {
      Serial.println();
    }
  }
}

// all channels
void playRTP(uint8_t intensity, uint16_t duration) {
  if (debug) {
    Serial.print("Playing intensity ");
    Serial.print(intensity);
    Serial.print(" for ");
    Serial.print(duration);
    Serial.println("ms on all channels");
  }
  for (uint8_t channel; channel < channelCount; channel++) {
    drvs[channel].setRealtimeValue(intensity);
  }
  delay(duration);
  for (uint8_t channel; channel < channelCount; channel++) {
    drvs[channel].setRealtimeValue(0x00);
  }
}

// single channel
void playRTP(uint8_t channel, uint8_t intensity, uint16_t duration) {
  if (debug) {
    Serial.print("Playing intensity ");
    Serial.print(intensity);
    Serial.print(" for ");
    Serial.print(duration);
    Serial.print("ms on channel ");
    Serial.println(channel);
  }
  drvs[channel].setRealtimeValue(intensity);
  delay(duration);
  drvs[channel].setRealtimeValue(0x00);
}
